@@ -385,7 +385,7 @@ foreach my $d (@sourceDir)
 	$prLog->print('-kind' => 'E',
 		      '-str' =>
 		      ["directory <$targetDir/$baseDir> already exists"]);
-	$exit = 1;
+	$exit = 1 unless $ignoreErrors;
     }    
 }
 exit $exit if $exit;
@@ -479,11 +479,14 @@ foreach my $d (@sourceDir)
     {
 	my ($mode, $uid, $gid, $atime, $mtime) = (stat($d))[2,4,5,8,9];
 
-	mkdir "$targetDir/$baseDir", $mode or
+	unless (mkdir "$targetDir/$baseDir", $mode)
+	{
 	    $prLog->print('-kind' => 'E',
 			  '-str' =>
-			  ["cannot create directory <$targetDir/$baseDir>"],
-			  '-exit' => 1);
+			  ["cannot create directory <$targetDir/$baseDir>"]);
+	    exit 1 unless $ignoreErrors;
+	}
+
 	chown $uid, $gid, "$targetDir/$baseDir";
 #	utime $atime, $mtime, "$targetDir/$baseDir";
 	my $wr = "$targetDir/$baseDir";
@@ -509,11 +512,13 @@ foreach my $d (@sourceDir)
 	{
 	    my ($mode, $uid, $gid, $atime, $mtime) =
 		(stat($f))[2,4,5,8,9];
-	    mkdir $fnew or
+	    unless (mkdir $fnew)
+	    {
 		$prLog->print('-kind' => 'E',
 			      '-str' =>
-			      ["cannot create directory <$fnew>"],
-			      '-exit' => 1);
+			      ["cannot create directory <$fnew>"]);
+		exit 1 unless $ignoreErrors;
+	    }
 	    chown $uid, $gid, "$fnew";
 	    my $wr = $fnew;
 	    $wr =~ s/\n/\0/og;
@@ -554,7 +559,7 @@ foreach my $d (@sourceDir)
 
 		    if ($t eq 'f')
 		    {
-			unless (&::copyFile($f, $fnew))
+			unless (&::copyFile($f, $fnew, $prLog))
 			{
 			    $prLog->print('-kind' => 'E',
 					  '-str' =>
@@ -612,7 +617,7 @@ foreach my $d (@sourceDir)
 
 			if ($t eq 'f')
 			{
-			    unless (&::copyFile($f, $fnew))
+			    unless (&::copyFile($f, $fnew, $prLog))
 			    {
 				$prLog->print('-kind' => 'E',
 					      '-str' =>
@@ -647,7 +652,7 @@ foreach my $d (@sourceDir)
 #print "\tcopy $f =>$fnew\n";
 		    if ($t eq 'f')
 		    {
-			unless (&::copyFile($f, $fnew))
+			unless (&::copyFile($f, $fnew, $prLog))
 			{
 			    $prLog->print('-kind' => 'E',
 					  '-str' =>
