@@ -414,7 +414,8 @@ my $parFork = parallelFork->new('-maxParallel' => $parJobs,
 my $tinySched = tinyWaitScheduler->new('-prLog' => $prLog);
 
 my $rcsf = undef;
-my ($meta, $postfix, $uncompr, @uncomprPar);
+my ($meta, $postfix, $uncompr, @uncomprPar,
+    $specialTypeArchiver, $archiveTypes);
 my $dirToCheck = undef;
 my $jobToDo = 1;
 my $parForkToDo = 0;
@@ -480,6 +481,7 @@ while (defined($rcsf) or $jobToDo > 0 or $parForkToDo > 0)
 		$prLog->print('-kind' => 'E',
 			      '-str' =>
 			      ["md5 sum mismatch for <$fn>"]);
+		$main::wft->print("$fn", 'md5Wrong');
 	    }
 	    elsif ($what eq 'errors')
 	    {
@@ -587,6 +589,10 @@ while (defined($rcsf) or $jobToDo > 0 or $parForkToDo > 0)
 	    $logInBackupDirFileName .= '.bz2'
 		if $compressLogInBackupDir eq 'yes';
 	     ($uncompr, @uncomprPar) = @{$rcsf->getInfoWithPar('uncompress')};
+	    $archiveTypes = $rcsf->getInfoWithPar('archiveTypes');
+	    $archiveTypes = '' unless $archiveTypes;
+	    $specialTypeArchiver =
+		$rcsf->getInfoWithPar('specialTypeArchiver');
 
 	    $filesFromMD5CheckSumFile{'.md5BlockCheckSums.bz2'} = 1;
 	    $filesFromMD5CheckSumFile{'.md5CheckSums.bz2'} = 1;
@@ -724,45 +730,54 @@ while (defined($rcsf) or $jobToDo > 0 or $parForkToDo > 0)
 		$prLog->print('-kind' => 'E',
 			      '-str' => ["named pipe <$filename is missing>"])
 		    unless -e $filename;
-		$prLog->print('-kind' => 'E',
-			      '-str' => ["<$filename> is not a named pipe!"])
-		    unless -p $filename;
-	    }
-	    elsif ($md5sum eq 'pipe')
-	    {
-		$prLog->print('-kind' => 'E',
-			      '-str' => ["named pipe <$filename is missing>"])
-		    unless -e $filename;
-		$prLog->print('-kind' => 'E',
-			      '-str' => ["<$filename> is not a named pipe!"])
-		    unless -p $filename;
+		unless ($specialTypeArchiver and
+		    $archiveTypes =~ /p/)
+		{
+		    $prLog->print('-kind' => 'E',
+				  '-str' => ["<$filename> is not a named pipe!"])
+			unless -p $filename;
+		}
 	    }
 	    elsif ($md5sum eq 'socket')
 	    {
 		$prLog->print('-kind' => 'E',
 			      '-str' => ["socket <$filename> is missing"])
 		    unless -e $filename;
-		$prLog->print('-kind' => 'E',
+		unless ($specialTypeArchiver and
+		    $archiveTypes =~ /S/)
+		{
+		    $prLog->print('-kind' => 'E',
 			      '-str' => ["<$filename> is not a socket!"])
 		    unless -S $filename;
+		}
 	    }
 	    elsif ($md5sum eq 'blockdev')
 	    {
 		$prLog->print('-kind' => 'E',
 			      '-str' => ["block device <$filename> is missing"])
 		    unless -e $filename;
-		$prLog->print('-kind' => 'E',
-			      '-str' => ["<$filename> is not a block device!"])
-		    unless -b $filename;
+		unless ($specialTypeArchiver and
+		    $archiveTypes =~ /p/)
+		{
+		    $prLog->print('-kind' => 'E',
+				  '-str' =>
+				  ["<$filename> is not a block device!"])
+			unless -b $filename;
+		}
 	    }
 	    elsif ($md5sum eq 'chardev')
 	    {
 		$prLog->print('-kind' => 'E',
 			      '-str' => ["char device <$filename> is missing"])
 		    unless -e $filename;
-		$prLog->print('-kind' => 'E',
-			      '-str' => ["<$filename> is not a char device!"])
-		    unless -c $filename;
+		unless ($specialTypeArchiver and
+		    $archiveTypes =~ /p/)
+		{
+		    $prLog->print('-kind' => 'E',
+				  '-str' =>
+				  ["<$filename> is not a char device!"])
+			unless -c $filename;
+		}
 	    }
 
 #print "-10-$f\n";
