@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2002,2004)
+#   Copyright (C) Dr. Heinz-Josef Claes (2002-2013)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -18,9 +18,6 @@
 #   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-my $VERSION = '$Id: storeBackupConvertBackup.pl 347 2008-11-04 15:29:59Z hjc $ ';
-push @VERSION, $VERSION;
-
 
 use IO::Handle;
 use strict;
@@ -90,25 +87,40 @@ require 'version.pl';
 
 my $md5CheckSums = '.md5CheckSums';
 
+=head1 NAME
 
-my $Help = <<EOH;
-converts old backups created with storeBackup.pl to the newest version,
-currently version 1.3
+storeBackupConvertBackup.pl - converts old backups created with storeBackup.pl to the newest version,
+
+=head1 SYNOPSIS
+
+storeBackupConvertBackup.pl storeBackup-dir
+
+=head1 DESCRIPTION
+
+This program converts old backups created with storeBackup.pl to the newest version,
+currently version 1.3.
 you can see the version by typing:
+
 head -1 < ...<storeBackupDir>/date_time/.md5CheckSums.info
-or if that file does not exist
+
+or if that file does not exist:
+
 bzip2 -d < ...<storeBackupDir>/date_time/.md5CheckSums.bz2 | head -1
+
 (if you do not see '###version=...', it is version 1.0
 
-usage:
-    $prog storeBackup-dir
+=head1 COPYRIGHT
 
-Copyright (c) 2002-2008 by Heinz-Josef Claes (see README)
+Copyright (c) 2002-2013 by Heinz-Josef Claes (see README)
 Published under the GNU General Public License v3 or any later version
-EOH
-    ;
 
-&printVersions(\@ARGV, '-V');
+=cut
+
+my $Help = join('', grep(!/^\s*$/, `pod2text "$0"`));
+$Help = "cannot find pod2text, see documentation for details\n"
+    unless $Help;
+
+&printVersion(\@ARGV, '-V');
 
 die $Help if (@ARGV != 1);
 
@@ -182,9 +194,9 @@ againForNextVersion:
 	    {
 		($uid, $gid, $mode) = (stat("$e/$md5CheckSums.bz2"))[4, 5, 2];
 		unlink "$e/$md5CheckSums.new.bz2", "$e/$md5CheckSums.info";
-		open(FILE, "bzip2 -d < \"$e/$md5CheckSums.bz2\" |") or
+		open(FILE, "bzip2 -d < \'$e/$md5CheckSums.bz2\' |") or
 		    die "cannot open $e/$md5CheckSums.bz2";
-		open(NEW, "| bzip2 > \"$e/$md5CheckSums.new.bz2\"") or
+		open(NEW, "| bzip2 > \'$e/$md5CheckSums.new.bz2\'") or
 		    die "cannot bzip2 > $e/$md5CheckSums.new.bz2";
 	    }
 	    else
@@ -256,7 +268,7 @@ againForNextVersion:
     {
 	if (-f "$e/$md5CheckSums.bz2")  # komprimierte Version liegt vor, nehmen
 	{
-	    open(FILE, "bzip2 -d < \"$e/$md5CheckSums.bz2\" |");
+	    open(FILE, "bzip2 -d < \'$e/$md5CheckSums.bz2\' |");
 	    $compress = 1;
 	}
 	elsif (-f $e/$md5CheckSums)
@@ -287,7 +299,7 @@ againForNextVersion:
 	    if ($compress == 1)
 	    {
 		unlink "$e/$md5CheckSums.new.bz2";
-		open(NEW, "| bzip2 > \"$e/$md5CheckSums.new.bz2\"") or
+		open(NEW, "| bzip2 > \'$e/$md5CheckSums.new.bz2\'") or
 		    die "cannot bzip2 > $e/$md5CheckSums.new.bz2";
 	    }
 	    else
@@ -344,7 +356,7 @@ againForNextVersion:
 	    if ($compress == 1)
 	    {
 		unlink "$e/$md5CheckSums.new.bz2", "$e/$md5CheckSums.info";
-		open(NEW, "| bzip2 > \"$e/$md5CheckSums.new.bz2\"") or
+		open(NEW, "| bzip2 > \'$e/$md5CheckSums.new.bz2\'") or
 		    die "cannot bzip2 > $e/$md5CheckSums.new.bz2";
 	    }
 	    else
