@@ -87,6 +87,7 @@ sub splitFileDir
 my ($req, $prog) = &libPath($0);
 push @INC, "$req";
 
+require 'storeBackupGlob.pl';
 require 'checkParam2.pl';
 require 'checkObjPar.pl';
 require 'prLog.pl';
@@ -99,6 +100,7 @@ require 'storeBackupLib.pl';
 my $md5CheckSumVersion = '1.1';
 my $noRestoreParallel = 12;
 my $checkSumFile = '.md5CheckSums';
+my $exit = 0;
 
 my $tmpdir = '/tmp';              # default value
 $tmpdir = $ENV{'TMPDIR'} if defined $ENV{'TMPDIR'};
@@ -351,18 +353,14 @@ my $rcsf = readCheckSumFile->new('-checkSumFile' =>
 my $fork = parallelFork->new('-maxParallel' => $noRestoreParallel,
 			     '-prLog' => $prLog);
 
-#my $meta = $rcsf->getMetaValField();
-
-#my ($uncompr, @uncomprPar) = @{$$meta{'uncompress'}};
 my ($uncompr, @uncomprPar) = @{$rcsf->getInfoWithPar('uncompress')};
-#my ($cp, @cpPar) = ('cp', '-dPR');
 my ($cp, @cpPar) = ('cp', '-dPR');
-#my $postfix = ($$meta{'postfix'})->[0];
 my $postfix = $rcsf->getInfoWithPar('postfix');
-#my $gnucp = ($$meta{'cpIsGnu'})->[0];
 my $gnucp = $rcsf->getInfoWithPar('cpIsGnu');
 $gnucp = ($gnucp eq 'yes') ? 1 : 0;
 $gnucp = 0 if $noGnuCp;
+my $archiveTypes = $rcsf->getInfoWithPar('archiveTypes');
+my $specialTypeArchiver = $rcsf->getInfoWithPar('specialTypeArchiver');
 
 $main::IOCompressDirect = 0;
 if ($uncompr eq 'bzip2' or $uncompr eq 'bunzip2')
@@ -448,6 +446,14 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 	{
 	    $targetFile = "$targetDir/$filename";
 	}
+	$targetFile =~ s/\/+/\//g;        # // -> /
+
+	if (not $overwrite and -e $targetFile)
+	{
+	    $prLog->print('-kind' => 'W',
+			  '-str' => ["target $targetFile already exists"]);
+	    next;
+	}
 
 	my $useGnuCp = $gnucp and ($md5sum eq 'socket' or
 				   $md5sum eq 'blockdev' or
@@ -499,6 +505,7 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 				      ["failed: link " .
 				       $DBMHardLink{$devInode} .
 				       " $targetFile"]);
+			$exit = 1;
 		    }
 		    goto contLoop;
 		}
@@ -508,58 +515,71 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 		}
 	    }
 	    my $linkTo = readlink "$backupRoot/$filename";
-	    if (not $overwrite and -e $targetFile)
-	    {
-		$prLog->print('-kind' => 'W',
-			      '-str' => ["target $targetFile already exists:",
-					 "\tln -s $linkTo $targetFile"]);
-	    }
-	    else
-	    {
-		++$noSymLinks;
-		symlink $linkTo, $targetFile;
 
-		# bei einigen Betriebssystem (z.B. Linux) wird bei Aufruf
-		# des Systemcalls chmod bei symlinks nicht der Symlink selbst
-		# geaendert, sondern die Datei, auf die er verweist.
-		# (dann muss lchown genommen werden -> Inkompatibilitaeten!?)
-		my $chown = forkProc->new('-exec' => 'chown',
-					  '-param' => ['-h', "$uid:$gid",
-						       "$targetFile"],
-					  '-outRandom' => "$tmpdir/chown-",
-					  '-prLog' => $prLog);
-		$chown->wait();
-#		utime $atime, $mtime, $targetFile;
-		$prLog->print('-kind' => 'I',
-			      '-str' => ["ln -s $linkTo $targetFile"])
-		    if ($verbose);
-	    }
+	    ++$noSymLinks;
+	    symlink $linkTo, $targetFile;
+
+	    # bei einigen Betriebssystem (z.B. Linux) wird bei Aufruf
+	    # des Systemcalls chmod bei symlinks nicht der Symlink selbst
+	    # geaendert, sondern die Datei, auf die er verweist.
+	    # (dann muss lchown genommen werden -> Inkompatibilitaeten!?)
+	    my $chown = forkProc->new('-exec' => 'chown',
+				      '-param' => ['-h', "$uid:$gid",
+						   "$targetFile"],
+				      '-outRandom' => "$tmpdir/chown-",
+				      '-prLog' => $prLog);
+	    $chown->wait();
+#	    utime $atime, $mtime, $targetFile;
+	    $prLog->print('-kind' => 'I',
+			  '-str' => ["ln -s $linkTo $targetFile"])
+		if ($verbose);
 	}
 	elsif ($md5sum eq 'pipe')
 	{
-	    my $mknod = forkProc->new('-exec' => 'mknod',
-				      '-param' => ["$targetFile", 'p'],
-				      '-outRandom' => "$tmpdir/mknod-",
-				      '-prLog' => $prLog);
-	    $mknod->wait();
-	    my $out = $mknod->getSTDOUT();
-	    $prLog->print('-kind' => 'E',
-			  '-str' =>
-			  ["STDOUT of <mknod $targetFile p>:", @$out])
-		if (@$out > 0);
-	    $out = $mknod->getSTDERR();
-	    $prLog->print('-kind' => 'E',
-			  '-str' =>
-			  ["STDERR of <mknod $targetFile p>:", @$out])
-		if (@$out > 0);
-	    chown $uid, $gid, $targetFile;
-	    chmod $mode, $targetFile;
-	    utime $atime, $mtime, $targetFile;
+	    if ($specialTypeArchiver and
+		$archiveTypes =~ /p/)
+	    {
+		&::extractFileFromArchive($filename, $targetDir, $backupRoot,
+		    $specialTypeArchiver, $prLog);
+		$prLog->print('-kind' => 'I',
+			      '-str' =>
+	     ["$specialTypeArchiver $backupRoot/$filename -> $targetFile"])
+		    if ($verbose);
+	    }
+	    else
+	    {
+		my $mknod = forkProc->new('-exec' => 'mknod',
+					  '-param' => ["$targetFile", 'p'],
+					  '-outRandom' => "$tmpdir/mknod-",
+					  '-prLog' => $prLog);
+		$mknod->wait();
+		my $out = $mknod->getSTDOUT();
+		$prLog->print('-kind' => 'E',
+			      '-str' =>
+			      ["STDOUT of <mknod $targetFile p>:", @$out])
+		    if (@$out > 0);
+		$out = $mknod->getSTDERR();
+		if (@$out > 0)
+		{
+		    $prLog->print('-kind' => 'E',
+				  '-str' =>
+				  ["STDERR of <mknod $targetFile p>:", @$out]);
+			$exit = 1;
+		}
+		chown $uid, $gid, $targetFile;
+		chmod $mode, $targetFile;
+		utime $atime, $mtime, $targetFile;
+		$prLog->print('-kind' => 'I',
+			      '-str' =>
+			      ["mknod $targetFile p"])
+		    if ($verbose);
+
+	    }
 	}
-	elsif (length($md5sum) == 32 or     # normal file
-	       $useGnuCp)                   # special file
+	elsif (length($md5sum) == 32 or            # normal file
+	       $useGnuCp or $specialTypeArchiver)  # special file
 	{
-# Idee zur Lsung des parallelitts-Prolems beim Zurcksichern
+# Idee zur Lsung des parallelitts-Problems beim Zurcksichern
 # in Verbindung mit dem Setzen der hard links:
 # erste Datei:
 # dev-inode => '.' in dbm-file (%DBMHardLink)
@@ -604,17 +624,32 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 		    $noBlockDevs++ if $md5sum eq 'blockdev';
 		    $noCharDevs++ if $md5sum eq 'chardev';
 
-		    ($old, $new) =
-			$fork->add_block('-exec' => $cp,
-				   '-param' => [@cpPar, "$backupRoot/$filename",
-						"$targetFile"],
-				   '-outRandom' => "$tmpdir/recover-",
-				   '-info' => [$targetFile, $uid, $gid, $mode,
-					       $atime, $mtime, $devInode]);
-		    $prLog->print('-kind' => 'I',
-				  '-str' =>
-				  ["cp $backupRoot/$filename $targetFile"])
-			if ($verbose);
+		    if ($specialTypeArchiver and
+			$archiveTypes =~ /[Sbc]/)
+		    {
+			&::extractFileFromArchive($filename, $targetDir, $backupRoot,
+						  $specialTypeArchiver, $prLog);
+
+			$prLog->print('-kind' => 'I',
+				      '-str' =>
+		      ["$specialTypeArchiver $backupRoot/$filename -> $targetFile"])
+			    if ($verbose);
+		    }
+		    else
+		    {
+			my $par = [@cpPar, "$backupRoot/$filename", "$targetFile"];
+			($old, $new) =
+			    $fork->add_block('-exec' => $cp,
+					     '-param' => $par,
+					     '-outRandom' => "$tmpdir/recover-",
+					     '-info' => [$targetFile, $uid, $gid, $mode,
+							 $atime, $mtime, $devInode,
+							 [$cp, @$par]]);
+			$prLog->print('-kind' => 'I',
+				      '-str' =>
+				      ["cp $backupRoot/$filename $targetFile"])
+			    if ($verbose);
+		    }
 		}
 	    }
 	    elsif ($compr eq 'c')          # war komprimiert
@@ -631,6 +666,9 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 		else
 		{
 		    ++$noFilesCompr;
+		    my $comm = [$uncompr, @uncomprPar, '<',
+				"$backupRoot/$filename$postfix", '>',
+				$targetFile];
 		    ($old, $new) =
 			$fork->add_block('-exec' => $uncompr,
 				   '-param' => \@uncomprPar,
@@ -639,7 +677,7 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 				   '-delStdout' => 'no',
 				   '-outRandom' => "$tmpdir/recover-",
 				   '-info' => [$targetFile, $uid, $gid, $mode,
-					       $atime, $mtime, $devInode]);
+					       $atime, $mtime, $devInode, $comm]);
 		    $prLog->print('-kind' => 'I',
 				  '-str' => ["$uncompr @uncomprPar < " .
 					     "$backupRoot/$filename$postfix > " .
@@ -650,13 +688,15 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 	    elsif ($compr eq 'b')       # blocked file
 	    {
 		++$noFilesBlocked;
+		my $comm = ["cp (block", "$backupRoot/$filename$postfix",
+			    "$targetFile"];
 		($old, $new) =
 		    $fork->add_block('-function' => \&uncompressCatBlock,
 				     '-funcPar' => ["$backupRoot/$filename",
 				     $targetFile, '\A\d.*', $uncompr, \@uncomprPar,
 				     $postfix, $prLog],
 				     '-info' => [$targetFile, $uid, $gid, $mode,
-						 $atime, $mtime, $devInode]);
+						 $atime, $mtime, $devInode, $comm]);
 		$prLog->print('-kind' => 'I',
 			      '-str' => ["cp (blocked) " .
 					     "$backupRoot/$filename$postfix " .
@@ -669,6 +709,7 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 			      '-str' =>
 			      ["unknow compr flag <$compr> in .md5CheckSums " .
 			       "for file <$backupRoot/$filename>"]);
+		$exit = 1;
 	    }
 	    if ($old)
 	    {
@@ -690,6 +731,7 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 	    $prLog->print('-kind' => 'E',
 			  '-str' =>
 			  ["unknown entry <$md5sum> for file <$filename>:"]);
+	    $exit = 1;
 	}
     }
 
@@ -705,8 +747,24 @@ finish:;
 my $job;
 while ($job = $fork->waitForAllJobs())
 {
-    my ($f, $oUid, $oGid, $oMode, $oAtime, $oMtime, $oDevInode) =
+    my ($f, $oUid, $oGid, $oMode, $oAtime, $oMtime, $oDevInode, $comm) =
 	@{$job->get('-what' => 'info')};
+
+    if (ref($job) eq 'forkProc')
+    {
+	my $out = $job->getSTDERR();
+	if (@$out > 0)
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' => ["STDERR of: @$comm", @$out]);
+	    $exit = 1;
+	}
+	if ($job->get('-what' => 'status'))
+	{
+	    $exit = 1;
+	}
+    }
+
     unless ($noHardLinks)
     {                                 # File in DBM vermerken
 	$DBMHardLink{$oDevInode} = $f;
@@ -730,6 +788,7 @@ unless (open(DIRFILE, '<', $tmpDirFile))
     $prLog->print('-kind' => 'E',
 		  '-str' => ["cannot read <$tmpDirFile>, cannot set " .
 			     "atime and mtime for directories"]);
+    $exit = 1;
 }
 else
 {
@@ -767,7 +826,7 @@ $prLog->print('-kind' => 'I',
     )
     if ($noRestored);
 
-exit 0;
+exit $exit;
 
 
 ############################################################
@@ -822,6 +881,7 @@ sub uncompressCatBlock
     my $prLog = shift;
 
     local *DIR;
+    my $ret = 0;
     unless (opendir(DIR, $fromDir))
     {
 	$prLog->print('-kind' => 'E',
@@ -842,6 +902,7 @@ sub uncompressCatBlock
     {
 	$prLog->print('-kind' => 'E',
 		      '-str' => ["cannot write to <$toFile>"]);
+	$ret = 1;
     }
 
     foreach $entry (sort @entries)
@@ -857,7 +918,12 @@ sub uncompressCatBlock
 		my $uc = new IO::Uncompress::Bunzip2 $input;
 		while ($uc->read($buffer, 10*1024**2))
 		{
-		    syswrite(TO, $buffer);
+		    unless (syswrite(TO, $buffer))
+		    {
+			$prLog->print('-kind' => 'E',
+				      '-str' => ["writing to <$toFile> failed"]);
+			$ret = 1;
+		    }
 		}
 		next;
 	    }
@@ -870,7 +936,12 @@ sub uncompressCatBlock
 				  '-prLog' => $prLog);
 	    while ($fileIn->sysread(\$buffer, 10*1024**2))
 	    {
-		syswrite(TO, $buffer);
+		unless (syswrite(TO, $buffer))
+		{
+		    $prLog->print('-kind' => 'E',
+				  '-str' => ["writing to <$toFile> failed"]);
+		    $ret = 1;
+		}
 	    }
 	}
 	else           # block not compressed
@@ -883,7 +954,12 @@ sub uncompressCatBlock
 	    }
 	    while (sysread(FROM, $buffer, 10*1024**2))
 	    {
-		syswrite(TO, $buffer);
+		unless (syswrite(TO, $buffer))
+		{
+		    $prLog->print('-kind' => 'E',
+				  '-str' => ["writing to <$toFile> failed"]);
+		    $ret = 1;
+		}
 	    }
 	}
 
@@ -898,5 +974,5 @@ sub uncompressCatBlock
 	}
     }
     close(TO);
-    return 0;
+    return $ret;
 }
