@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2001-2012)
+#   Copyright (C) Heinz-Josef Claes (2001-2013)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 
 use Digest::MD5 qw(md5_hex);
 use Fcntl qw(O_RDWR O_CREAT);
+use Fcntl ':mode';
 use POSIX;
 
 require 'prLog.pl';
@@ -39,6 +40,7 @@ sub copyFile
 {
     my $source = shift;
     my $target = shift;
+    my $prLog = shift;
 
     local *SOURCE;
     local *TARGET;
@@ -53,7 +55,12 @@ sub copyFile
     my $buffer;
     while (sysread(SOURCE, $buffer, 10*1024**2))
     {
-	syswrite(TARGET, $buffer);
+	unless (syswrite(TARGET, $buffer))
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' => [$!]);
+	    return 0;
+	}
     }
 
     close(TARGET);
@@ -515,6 +522,12 @@ sub checkLockFile
 			  '-str' => ["removing old lock file of process <$pid>"]
 			  );
 	}
+	unless (unlink $lockFile)
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' => ["cannot remove lock file <$lockFile"],
+			  '-exit' => 1);
+	}
     }
 
     $prLog->print('-kind' => 'I',
@@ -570,6 +583,170 @@ sub readSymLinkCalcMd5
     return $md5->hexdigest();
 }
 
+############################################################
+sub createArchiveFromFile
+{
+    my $filename = shift;   # relative path
+    my $sourceDir = shift;
+    my $backupDir = shift;
+    my $archiver = shift;
+    my $prLog = shift;
+
+    my $prog = $main::fileTypeArchiver{$archiver}{'prog'};
+    my $opts = $main::fileTypeArchiver{$archiver}{'createOpts'}; # \@list
+    my ($pathToSourceFile, $f) = &::splitFileDir("$sourceDir/$filename");
+
+    if ($archiver eq 'tar')
+    {
+	push @$opts, '-', $f;
+	my $tar = forkProc->new('-exec' => $prog,
+				'-param' => $opts,
+				'-workingDir' => $pathToSourceFile,
+				'-stdout' => "$backupDir/$filename",
+				'-stderr' => '/tmp/stbuPipeArchive-',
+				'-delStdout' => 'no',
+				'-prLog' => $prLog);
+	$tar->wait();
+	my $out = $tar->getSTDOUT();
+	$prLog->print('-kind' => 'W',
+		      '-str' => ["STDOUT of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+	$out = $tar->getSTDERR();
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["STDERR of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+    }
+    elsif ($archiver eq 'cpio')
+    {
+	my $cpio = pipeToFork->new('-exec' => $prog,
+				   '-param' => $opts,
+				   '-workingDir' => $pathToSourceFile,
+				   '-stdout' => "$backupDir/$filename",
+				   '-stderr' => '/tmp/stbuPipeArchive-',
+				   '-delStdout' => 'no',
+				   '-prLog' => $prLog);
+	$cpio->print("$f\n");
+	$cpio->wait();
+	my $out = $cpio->getSTDERR();
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["STDERR of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+    }
+    else
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["archiver <$archiver> unknown, exiting"],
+		      '-exit' => 1);
+    }
+}
+
+
+############################################################
+sub extractFileFromArchive
+{
+    my $filename = shift;   # relative path
+    my $restoreDir = shift;
+    my $backupDir = shift;
+    my $archiver = shift;
+    my $prLog = shift;
+
+#print "+++extractFileFromArchive:+$filename+$restoreDir+$backupDir+$archiver+\n";
+    my $prog = $main::fileTypeArchiver{$archiver}{'prog'};
+    my $opts = $main::fileTypeArchiver{$archiver}{'extractOpts'}; # \@list
+    my ($pathToRestoreDir, $f) = &::splitFileDir("$restoreDir/$filename");
+
+    if ($archiver eq 'tar')
+    {
+	push @$opts, "$backupDir/$filename";
+#print "+++$prog @$opts+++\n";
+	my $tar = forkProc->new('-exec' => $prog,
+				'-param' => $opts,
+				'-workingDir' => $pathToRestoreDir,
+				'-outRandom' => '/tmp/stbuPipeArchive-',
+				'-prLog' => $prLog);
+	$tar->wait();
+	my $out = $tar->getSTDOUT();
+	$prLog->print('-kind' => 'W',
+		      '-str' => ["STDOUT of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+	$out = $tar->getSTDERR();
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["STDERR of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+    }
+    elsif ($archiver eq 'cpio')
+    {
+#print "+++$prog @$opts+++\n";
+	my $cpio = forkProc->new('-exec' => $prog,
+				 '-param' => $opts,
+				 '-workingDir' => $pathToRestoreDir,
+				 '-stdin' => "$backupDir/$filename",
+				 '-outRandom' => '/tmp/stbuPipeArchive-',
+				 '-prLog' => $prLog);
+	$cpio->wait();
+	my $out = $cpio->getSTDOUT();
+	$prLog->print('-kind' => 'W',
+		      '-str' => ["STDOUT of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+	$out = $cpio->getSTDERR();
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["STDERR of <$prog @$opts>:", @$out])
+	    if (@$out > 0);
+    }
+    else
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["archiver <$archiver> unknown, exiting"],
+		      '-exit' => 1);
+    }
+#print "+++end of extractFileFromArchive:+$filename+\n";
+}
+
+
+############################################################
+# helper functions for &main::MARK_DIR and &main::MARK_DIR_REC
+# problem is to store 2 dimensional hash with dbm files (saveRAM)
+# simulates $a{$b}{$c} = 1 or $a{$b}{$d} = 1
+# with $a{$b} = 'SEP$cSEP$aSEP' where SEPerator is \000
+sub addToStr
+{
+    my $hash = shift;
+    my $k1 = shift;
+    my $k2 = shift;
+
+    if (exists $$hash{$k1})
+    {
+	if (index($$hash{$k1}, "\000$k2\000") < 0)  # not found
+	{
+	    $$hash{$k1} = $$hash{$k1} . "$k2\000";
+	}
+    }
+    else
+    {
+	$$hash{$k1} = "\000$k2\000";
+    }
+}
+
+########################################
+sub existStr
+{
+    my $hash = shift;
+    my $k1 = shift;
+    my $k2 = shift;
+
+#    print "\thash = ", $$hash{$k1}, "\n";
+#    print "\t\t $k1 exists\n" if (exists $$hash{$k1});
+
+    if (exists $$hash{$k1} and index($$hash{$k1}, "\000$k2\000") >= 0)
+    {
+	return 1;
+    }
+    else
+    {
+	return 0;
+    }
+}
+
 
 ############################################################
 # Objekt kann zum (wiederholten) Abfragen von Informationen
@@ -875,6 +1052,9 @@ sub readDir
     {
 	next if ($entry eq '.' or $entry eq '..');
 	$entry = $dir . '/' . $entry;
+
+#	my $mode = (stat($entry))[2];
+#	if (::S_ISLNK($mode))
 	if (-l $entry)
 	{
 	    if ($self->{'followLinks'} > $depth and
@@ -902,7 +1082,11 @@ sub readDir
 	    next;
 	}
 	unless (-r $entry)
+#	unless ($mode & ::S_IREAD and $> != 0)  # $> -> effective user id
 	{
+	    next
+		if exists $$except{&::absolutePath($entry)};
+
 	    if ($ignoreReadError)
 	    {
 		$prLog->print('-kind' => $prLogWarn,
@@ -917,6 +1101,7 @@ sub readDir
 	    next;
 	}
 	if (-d $entry)       # Dieses Directory mu beim Kopieren
+#	if (::S_ISDIR($mode))  # Dieses Directory mu beim Kopieren
 	{                      # z.B. mu angelegt werden!
 	    push @$files, $entry;
 	    push @$types, 'd';
@@ -926,30 +1111,35 @@ sub readDir
 	    next;
 	}
 	if (-f $entry)
+#	if (::S_ISREG($mode))
 	{
 	    push @$files, $entry;
 	    push @$types, 'f';
 	    next;
 	}
 	if (-p $entry)      # named pipe
+#	if (::S_ISFIFO($mode))      # named pipe
 	{
 	    push @$files, $entry;
 	    push @$types, 'p';
 	    next;
 	}
 	if (-S $entry)      # socket
+#	if (::S_ISSOCK($mode))      # socket
 	{
 	    push @$files, $entry;
 	    push @$types, 'S';
 	    next;
 	}
 	if (-b $entry)      # block special file
+#	if (::S_ISBLK($mode))      # block special file
 	{
 	    push @$files, $entry;
 	    push @$types, 'b';
 	    next;
 	}
 	if (-c $entry)      # character special file
+#	if (::S_ISCHR($mode))      # character special file
 	{
 	    push @$files, $entry;
 	    push @$types, 'c';
